package biz.capitalware.mqfm.action.handlers;

import java.util.ArrayList;

import biz.capitalware.mqfm.MQFM;
import biz.capitalware.mqfm.action.objects.*;
import biz.capitalware.mqfm.actions.*;

/**
 * This class will process a list of action commands.
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class ProcessActions
{
   /**
    * Make sure we don't call it.
    */
   private ProcessActions()
   {}

   /**
    * Loop through a list of action commands and process each one.
    * @param actionAL
    * @return exitCode
    */
   public static int doWork(ArrayList<FMObject> actionAL)
   {
      int      exitCode = 0;
      FMObject fmo = null;

      /**
       * First check to make sure all of the objects are ok.
       */
      try
      {
         for (int i=0; i < actionAL.size(); i++)
         {
            fmo = actionAL.get(i);
            fmo.performTokenization();
            fmo.checkObject();
         }
      }
      catch (IllegalArgumentException e)
      {
         MQFM.logger.fatal("IllegalArgumentException: " + fmo.actionName + " Action : " + e.getLocalizedMessage());
         MQFM.globalErrorText = fmo.actionName + " Action : " + e.getLocalizedMessage();
         exitCode = 16;
      }
      catch (Exception e)
      {
         MQFM.logger.fatal("Exception: " + fmo.actionName + " Action : " + e.getLocalizedMessage());
         MQFM.globalErrorText = fmo.actionName + " Action : " + e.getLocalizedMessage();
         exitCode = 16;
      }

      /**
       * If there was an issue in the checkObject method, do not continue
       */
      if (exitCode != 0)
         return exitCode;

      /**
       * Life's good - now process each action command.
       */
      try
      {
         for (int i=0; (i < actionAL.size()) && (exitCode == 0); i++)
         {
            MQFM.globalErrorText = null;
            fmo = actionAL.get(i);

            if (fmo instanceof SendObject)
            {
               SendAction sa = new SendAction((SendObject) fmo);
               exitCode = sa.doWork();
            }
            else if (fmo instanceof WatchObject)
            {
               WatchAction wa = new WatchAction((WatchObject) fmo);
               exitCode = wa.doWork();
            }
            else if (fmo instanceof ReceiveObject)
            {
               ReceiveObject ra = (ReceiveObject) fmo;
               ReceiveAction mf = new ReceiveAction(ra);
               exitCode = mf.doWork();
            }
            else if (fmo instanceof PutQuitObject)
            {
               PutQuitAction pqa = new PutQuitAction((PutQuitObject)fmo);
               exitCode = pqa.doWork();
            }
            else if (fmo instanceof ExecuteObject)
            {
               ExecuteAction ea = new ExecuteAction((ExecuteObject)fmo);
               exitCode = ea.doWork();
            }
            else if (fmo instanceof IfObject)
            {
               IfAction ea = new IfAction((IfObject)fmo);
               exitCode = ea.doWork();
            }
            else if (fmo instanceof LaunchObject)
            {
               LaunchAction la = new LaunchAction((LaunchObject)fmo);
               exitCode = la.doWork();
            }
            else if (fmo instanceof ScheduleObject)
            {
               ScheduleAction sa = new ScheduleAction((ScheduleObject)fmo);
               exitCode = sa.doWork();
            }
            else if (fmo instanceof SendEmailObject)
            {
               SendEmailAction sea = new SendEmailAction((SendEmailObject)fmo);
               exitCode = sea.doWork();
            }
            else if (fmo instanceof SleepObject)
            {
               SleepAction sa = new SleepAction((SleepObject)fmo);
               exitCode = sa.doWork();
            }
            else if (fmo instanceof AppendObject)
            {
               AppendAction aa = new AppendAction((AppendObject)fmo);
               exitCode = aa.doWork();
            }
            else if (fmo instanceof CopyObject)
            {
               CopyAction ca = new CopyAction((CopyObject)fmo);
               exitCode = ca.doWork();
            }
            else if (fmo instanceof DecryptFileObject)
            {
               DecryptFileAction dfa = new DecryptFileAction((DecryptFileObject)fmo);
               exitCode = dfa.doWork();
            }
            else if (fmo instanceof DeleteObject)
            {
               DeleteAction da = new DeleteAction((DeleteObject)fmo);
               exitCode = da.doWork();
            }
            else if (fmo instanceof EncryptFileObject)
            {
               EncryptFileAction efa = new EncryptFileAction((EncryptFileObject)fmo);
               exitCode = efa.doWork();
            }
            else if (fmo instanceof MergeObject)
            {
               MergeAction ma = new MergeAction((MergeObject)fmo);
               exitCode = ma.doWork();
            }
            else if (fmo instanceof MergeSortObject)
            {
               MergeSortAction msa = new MergeSortAction((MergeSortObject)fmo);
               exitCode = msa.doWork();
            }
            else if (fmo instanceof MoveObject)
            {
               MoveAction ma = new MoveAction((MoveObject)fmo);
               exitCode = ma.doWork();
            }
            else if (fmo instanceof RenameObject)
            {
               RenameAction ra = new RenameAction((RenameObject)fmo);
               exitCode = ra.doWork();
            }
            else if (fmo instanceof ReplaceTextObject)
            {
               ReplaceTextAction rta = new ReplaceTextAction((ReplaceTextObject)fmo);
               exitCode = rta.doWork();
            }
            else if (fmo instanceof SortObject)
            {
               SortAction sa = new SortAction((SortObject)fmo);
               exitCode = sa.doWork();
            }
            else if (fmo instanceof TarObject)
            {
               TarAction ta = new TarAction((TarObject)fmo);
               exitCode = ta.doWork();
            }
            else if (fmo instanceof TouchObject)
            {
               TouchAction ta = new TouchAction((TouchObject)fmo);
               exitCode = ta.doWork();
            }
            else if (fmo instanceof UnTarObject)
            {
               UnTarAction uta = new UnTarAction((UnTarObject)fmo);
               exitCode = uta.doWork();
            }
            else if (fmo instanceof UnZipObject)
            {
               UnZipAction uza = new UnZipAction((UnZipObject)fmo);
               exitCode = uza.doWork();
            }
            else if (fmo instanceof ZipObject)
            {
               ZipAction za = new ZipAction((ZipObject)fmo);
               exitCode = za.doWork();
            }
            else
            {
               MQFM.logger.fatal("Unknown action selected: " + fmo.toString());
               exitCode = 99;
            }
            
            MQFM.actionObjectValues.put(fmo.labelName+"_"+fmo.actionName, fmo);
         }
      }
      catch (Exception e)
      {
         MQFM.logger.fatal("Exception: " + fmo.actionName + " Action : " + e.getLocalizedMessage());
         exitCode = 16;
      }

      return exitCode;
   }
}
