package biz.capitalware.mqfm.action.objects;

import java.io.File;
import java.util.ArrayList;

import com.ibm.mq.*;

import biz.capitalware.mqfm.MQFM;
import biz.capitalware.mqfm.handlers.IO;
import biz.capitalware.mqfm.handlers.Tokens;
import biz.capitalware.mqfm.objects.Constants;
import biz.capitalware.mqfm.objects.Cryptography;

/**
 * This class holds the values for the Send action.
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class SendObject extends FMFileMQObject
{
   public String           srcFilename = "";
   public char             format = ' ';
   public String           jobXmlFile = null;
   public String           remoteJobXmlFile = null;
   public String           remoteDirectory = null;
   public String           remoteFilename = null;
   public String           archiveDirectory = Constants.ARCHIVE_PATH;
   public String           archiveFileName = null;
   public int              priority = MQC.MQPRI_PRIORITY_AS_Q_DEF;
   public boolean          persistence = true;
   public boolean          deleteFile = false;
   public boolean          archiveFile = false;
   public boolean          archiveCreateDir = false;
   public Cryptography     crypto = null;

   /**
    * The constructor
    */
   public SendObject()
   {
      super();
   }

   /**
    * Process tokens in Action Object's variables
    */
   public void performTokenization()
   {
      if (srcFilename != null)
         srcFilename = Tokens.replace(srcFilename, MQFM.userProperties);
      if (jobXmlFile != null)
         jobXmlFile = Tokens.replace(jobXmlFile, MQFM.userProperties);
      if (remoteDirectory != null)
         remoteDirectory = Tokens.replace(remoteDirectory, MQFM.userProperties);
      if (remoteFilename != null)
         remoteFilename = Tokens.replace(remoteFilename, MQFM.userProperties);
      if (remoteJobXmlFile != null)
         remoteJobXmlFile = Tokens.replace(remoteJobXmlFile, MQFM.userProperties);
      if (archiveDirectory != null)
         archiveDirectory = Tokens.replace(archiveDirectory, MQFM.userProperties);
      if (archiveFileName != null)
         archiveFileName = Tokens.replace(archiveFileName, MQFM.userProperties);
   }

   /**
    * Check the user specified parameters.
    */
   public void checkObject()
   {
      if (remoteDirectory != null)
      {
         if ( remoteDirectory.startsWith("jobs"))
            throw new IllegalArgumentException("Security risk. Remote directory cannot be the \"jobs\" directory.");
      }

      if ( ("".equals(srcFilename)) || (srcFilename == null) )
         throw new IllegalArgumentException("Missing a required tag: File");
      else
      {
         if (srcFilename.indexOf('*') != -1)
         {
            if (srcFilename.indexOf('*') != srcFilename.lastIndexOf('*'))
               throw new IllegalArgumentException("Invalid filename wildcard: '"+srcFilename+"'. Send action only supports a single wildcard '*'. i.e. '*.txt' or 'tr*' or 'ab*.txt'");
         }
         else
         {
            File pfn = new File(srcFilename);
            if (!(pfn.exists()))
                  throw new IllegalArgumentException("InputFileName '"+srcFilename+"' does not exit.");
         }
      }

      if ( deleteFile &&  archiveFile)
         throw new IllegalArgumentException("Error: User cannot specify both delete and archive together.");

      if (jobXmlFile != null)
      {
         File ljn = new File(Constants.JOBS_PATH + jobXmlFile);
         if (!(ljn.exists()))
               throw new IllegalArgumentException("JobFile '"+jobXmlFile+"' does not exit.");
      }

      if (crypto != null)
      {
         if ( (crypto.keySize != 128) && (crypto.keySize != 192) && (crypto.keySize != 256) )
            throw new IllegalArgumentException("Error: You must specify a keysize attribute that is 128 or 192 or 256 for the Send action.");

         if ("".equals(crypto.passPhrase))
            throw new IllegalArgumentException("Error: PassPhrase attribute cannot be blank for the Send action.");
      }

      /* set default values */
      String curDir = Constants.HOME_PATH;
      String filename = srcFilename;

      if (srcFilename.lastIndexOf(File.separator) != -1)
      {
         curDir = IO.getPath(srcFilename);
         filename = IO.getFileName(srcFilename);
      }

      if (remoteDirectory == null)
         remoteDirectory = curDir;

      if (remoteFilename == null)
         remoteFilename = filename;
   }

   /**
    * Output the info related to Send Action
    * @param logger
    */
   public void printValues()
   {
      MQFM.logger.info("     Format               = " + format);
      MQFM.logger.info("     Priority             = " + (priority == MQC.MQPRI_PRIORITY_AS_Q_DEF? "MQPRI_PRIORITY_AS_Q_DEF" : priority));
      MQFM.logger.info("     Persistence          = " + (persistence? "Yes":"No"));
      MQFM.logger.info("     Input Path & Filename= " + srcFilename);
      MQFM.logger.info("     JobFile              = " + (jobXmlFile == null ? "" : jobXmlFile));
      MQFM.logger.info("     Delete File          = " + (deleteFile?"Yes":"No"));
      MQFM.logger.info("     Archive File         = " + (archiveFile?"Yes":"No"));
      MQFM.logger.info("     Archive Create Dir   = " + (archiveCreateDir?"Yes":"No"));
      MQFM.logger.info("     Archive Directory    = " + archiveDirectory);
      MQFM.logger.info("     Archive Filename     = " + (archiveFileName == null ? "" : archiveFileName));
      MQFM.logger.info("     Remote JobXmlFile    = " + (remoteJobXmlFile == null ? "" : remoteJobXmlFile));
      MQFM.logger.info("     Remote Directory     = " + remoteDirectory);
      MQFM.logger.info("     Remote Filename      = " + remoteFilename);
      if (crypto != null)
      {
         MQFM.logger.info("     KeySize              = " + crypto.keySize);
         MQFM.logger.info("     PassPhrase           = " + crypto.passPhrase );
      }
   }

   /**
    * Output the info related to Send Action
    * @param logger
    */
   public void printValues(boolean showFileNames)
   {
      MQFM.logger.info("     Format               = " + format);
      MQFM.logger.info("     Priority             = " + (priority == MQC.MQPRI_PRIORITY_AS_Q_DEF? "MQPRI_PRIORITY_AS_Q_DEF" : priority));
      MQFM.logger.info("     Persistence          = " + (persistence? "Yes":"No"));
      if (showFileNames)
         MQFM.logger.info("     Input Path & Filename= " + srcFilename);
      MQFM.logger.info("     JobFile              = " + (jobXmlFile == null ? "" : jobXmlFile));
      MQFM.logger.info("     Delete File          = " + (deleteFile?"Yes":"No"));
      MQFM.logger.info("     Archive File         = " + (archiveFile?"Yes":"No"));
      MQFM.logger.info("     Archive Create Dir   = " + (archiveCreateDir?"Yes":"No"));
      MQFM.logger.info("     Archive Directory    = " + archiveDirectory);
      MQFM.logger.info("     Archive Filename     = " + (archiveFileName == null ? "" : archiveFileName));
      MQFM.logger.info("     Remote JobXmlFile    = " + (remoteJobXmlFile == null ? "" : remoteJobXmlFile));
      MQFM.logger.info("     Remote Directory     = " + remoteDirectory);
      if (showFileNames)
         MQFM.logger.info("     Remote Filename      = " + remoteFilename);
      if (crypto != null)
      {
         MQFM.logger.info("     KeySize              = " + crypto.keySize);
         MQFM.logger.info("     PassPhrase           = " + crypto.passPhrase );
      }
   }
   
   /**
    * Convert the object to a string.
    * @return s
    */
   public ArrayList<String> toStringArray()
   {
      ArrayList<String> al = new ArrayList<String>();
      String file;

      if (crypto != null)
         al.add("<"+Constants.SEND_ELEMENT+" "+Constants.DELETE_ATTRIBUTE+"=\""+(deleteFile?"Y":"N")+"\" "+Constants.FORMAT_ATTRIBUTE+"=\""+format+"\" "+Constants.PRIORITY_ATTRIBUTE+"=\""+priority+"\" "+Constants.PERSISTENCE_ATTRIBUTE+"=\""+(persistence? "Y":"N")+"\" "+Constants.KEY_SIZE_ATTRIBUTE+"=\""+crypto.keySize+"\" "+Constants.PASS_PHRASE_ATTRIBUTE+"=\""+crypto.passPhrase+"\">"+LINE_SEPARATOR);
      else
         al.add("<"+Constants.SEND_ELEMENT+" "+Constants.DELETE_ATTRIBUTE+"=\""+(deleteFile?"Y":"N")+"\" "+Constants.FORMAT_ATTRIBUTE+"=\""+format+"\" "+Constants.PRIORITY_ATTRIBUTE+"=\""+priority+"\" "+Constants.PERSISTENCE_ATTRIBUTE+"=\""+(persistence? "Y":"N")+"\">"+LINE_SEPARATOR);
      
      if (srcFilename.indexOf(File.separator) != -1 )
      {
         int x = srcFilename.lastIndexOf(File.separator);
         
         String dir   = srcFilename.substring(0, x);
         String fname = srcFilename.substring(x+1);
         
         file = "  <"+Constants.FILE_ELEMENT+" "+Constants.DIR_ATTRIBUTE+"=\""+dir+"\">"+fname+"</"+Constants.FILE_ELEMENT+">";
      }
      else
         file = "  <"+Constants.FILE_ELEMENT+">"+srcFilename+"</"+Constants.FILE_ELEMENT+">";
      
      al.add(file+LINE_SEPARATOR);

      /*
       * MQ info
       */
      ArrayList<String> mq = createMQTag();
      for (int i=0; i < mq.size(); i++)
         al.add(mq.get(i));

      /*
       * Remote info
       */
      ArrayList<String> rt = createRemoteTag(remoteJobXmlFile, remoteDirectory, remoteFilename);
      for (int i=0; i < rt.size(); i++)
         al.add(rt.get(i));
      
      if (jobXmlFile != null)
         al.add("  <"+Constants.EXECUTE_ELEMENT+" "+Constants.XML_FILE_ATTRIBUTE+"=\""+jobXmlFile+"\" />"+LINE_SEPARATOR);
      
      if (archiveFile)
         al.add("  <"+Constants.ARCHIVE_ELEMENT+" "+Constants.CREATE_DIR_ATTRIBUTE+"=\""+(archiveCreateDir?'Y':'N')+"\" "+" "+Constants.TO_DIR_ATTRIBUTE+"=\""+archiveDirectory+"\" "+Constants.TO_FILE_ATTRIBUTE+"=\""+archiveFileName+"\" />"+LINE_SEPARATOR);
      
      al.add("    </"+Constants.SEND_ELEMENT+">"+LINE_SEPARATOR);
      
      return al;
   }
}
