package biz.capitalware.mqfm.actions;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Properties;

import javax.mail.Authenticator;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.NoSuchProviderException;
import javax.mail.PasswordAuthentication;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import org.jdom.JDOMException;

import biz.capitalware.mqfm.MQFM;
import biz.capitalware.mqfm.action.objects.SendEmailObject;
import biz.capitalware.mqfm.handlers.Tokens;
import biz.capitalware.mqfm.xml.EmailXml;

/**
 * This class will handle the SendEmail action.
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class SendEmailAction extends FMAction
{
   private     SendEmailObject   seo;
   protected   EmailXml          exh;

   /**
    * The constructor
    * @param seo
    * @param logger
    */
   public SendEmailAction(SendEmailObject seo)
   {
      super(seo.actionName);
      this.seo = seo;
   }

   /**
    * Perform the work.
    */
   public int doWork()
   {
      int exitCode = 0;
      seo.printValues();
      Session mailSession = null;
      Transport transport = null;
      
      exh = new EmailXml();
      try
      {
         exh.readXMLFile(seo.xmlFile);
         
         Properties props = new Properties();
         props.put("mail.transport.protocol", "smtp");
         props.put("mail.smtp.host", exh.getSmtpHost());
         props.put("mail.smtp.port", exh.getSmtpPort());
         
         if (exh.isSsl())
         {
            props.put("mail.smtp.starttls.enable", "true");
            props.put("mail.smtp.socketFactory.class", "javax.net.ssl.SSLSocketFactory");
            props.put("mail.smtp.socketFactory.fallback", "false");
         }
         
         if (exh.isAuth())
         {
            props.put("mail.smtp.auth", "true");
            Authenticator auth = new SMTPAuthenticator();
            mailSession = Session.getInstance(props, auth);
         }
         else
            mailSession = Session.getInstance(props, null);
         
         // uncomment for debugging info to stdout
         // mailSession.setDebug(true);
         
         transport = mailSession.getTransport();
         transport.connect();
         MQFM.logger.info("Connected to " + exh.getSmtpHost()+ " on port # " + exh.getSmtpPort());
         
         MimeMessage message = new MimeMessage(mailSession);
         message.setSentDate(new Date());
         
         /*
          * Do token replacement on the subject field
          */
         String subject = Tokens.replace(seo.actionName, exh.getSubject(), MQFM.userProperties);
         if (MQFM.globalErrorText == null)
            subject = Tokens.replace(subject, new StringBuffer(""));
         else
            subject = Tokens.replace(subject, new StringBuffer(MQFM.globalErrorText));
         message.setSubject(subject);
         
         /*
          * Do token replacement on the message text field
          */
         String msgText = Tokens.replace(seo.actionName, exh.getMessageText(), MQFM.userProperties);
         if (MQFM.globalErrorText == null)
            msgText = Tokens.replace(msgText, new StringBuffer(""));
         else
            msgText = Tokens.replace(msgText, new StringBuffer(MQFM.globalErrorText));
         message.setContent(msgText, "text/plain");
         
         message.setFrom(new InternetAddress(exh.getFromEmailAddress()));

         /*
          * Set 'TO' address
          */
         ArrayList<String> toEmail = exh.getToAddress();
         InternetAddress[] addressTo = new InternetAddress[toEmail.size()];
         for (int i = 0; i < toEmail.size(); i++)
            addressTo[i] = new InternetAddress(toEmail.get(i));

         message.setRecipients(Message.RecipientType.TO, addressTo);
         
         transport.sendMessage(message, message.getRecipients(Message.RecipientType.TO));

         seo.msgCount++;

         MQFM.logger.info("Sent email to "+exh.getToAddress().toString());
      }
      catch (NoSuchProviderException e)
      {
         MQFM.logger.fatal("NoSuchProviderException: " + e.getLocalizedMessage());
         if (e.getCause() != null)
            MQFM.logger.fatal(e.getCause().toString());;
         exitCode = 8;
      }
      catch (MessagingException e)
      {
         MQFM.logger.fatal("MessagingException: " + e.getLocalizedMessage());
         if (e.getCause() != null)
            MQFM.logger.fatal(e.getCause().toString());;
         exitCode = 8;
      }
      catch (JDOMException e)
      {
         MQFM.logger.fatal("JDOMException: " + e.getLocalizedMessage());
         MQFM.globalErrorText = e.getLocalizedMessage();
         exitCode = 8;
      }
      catch (IOException e)
      {
         MQFM.logger.fatal("IOException: " + e.getLocalizedMessage());
         MQFM.globalErrorText = e.getLocalizedMessage();
         exitCode = 8;
      }
      finally
      {
         try
         {
            if (transport != null)
               transport.close();
         }
         catch (MessagingException e)
         {
            MQFM.logger.fatal("MessagingException: " + e.getLocalizedMessage());
            if (e.getCause() != null)
               MQFM.logger.fatal(e.getCause().toString());;
            exitCode = 8;
         }
         MQFM.logger.info("Disconnected." );
      }

      if (!seo.onErrorFail)
         exitCode = 0; /* reset back to zero */
      
      return exitCode;
   }

   /**
    * Class to setup UserID and Password for connection to SMTP host. 
    */
   private class SMTPAuthenticator extends javax.mail.Authenticator
   {
      public PasswordAuthentication getPasswordAuthentication()
      {
         return new PasswordAuthentication(exh.getSmtpUserID(), exh.getSmtpPassword());
      }
   }
}
