package biz.capitalware.mqfm.handlers;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.RandomAccessFile;
import java.net.URL;
import java.nio.channels.Channels;
import java.nio.channels.FileChannel;
import java.nio.channels.FileLock;
import java.nio.channels.ReadableByteChannel;
import java.util.ArrayList;
import java.util.Date;

import biz.capitalware.mqfm.MQFM;
import biz.capitalware.mqfm.objects.MQFMException;

/**
 * This class will handle all of MQFM's IO work.
 * @author Roger Lacroix
 * @version 1.1
 * @license Apache 2 License
 */
public class IO
{
   /**
    * The constructor - make sure nothing instantiates this class.
    * It must be accessed in a static way.
    */
   private IO()
   {}

   /**
    * This method will read a file.
    *
    * @param fileName the name of the file.
    * @param format MQFM message format
    * @param logger the Log4J logger
    * @return byte array of the contents of the file.
    * @throws MQFMException
    */
   public static byte[] readFile(String fileName) throws MQFMException
   {
      RandomAccessFile in = null;
      FileChannel fileChannel = null;
      FileLock fileLock = null;
      boolean failed = false;
      byte[] byteData = null;
      MQFM.logger.debug("readFile()");

      try
      {
         in = new RandomAccessFile(fileName, "r");
         long len = in.length();
         if (MQFM.useFileLocking)
         {
            fileChannel = in.getChannel();
            fileLock = fileChannel.tryLock(0, len, true); /* shared lock for reading */
         }

         if ( ( (fileLock != null) && (MQFM.useFileLocking) ) ||
               (!MQFM.useFileLocking) )
         {
            byteData = new byte[(int)len];

            in.read(byteData);

            MQFM.logger.info("Read completed.");
         }
         else
         {
            MQFM.logger.fatal("IO.readFile cannot acquire lock for file: " + fileName);
            MQFM.globalErrorText = "IO.readFile cannot acquire lock for file: " + fileName;
         }
      }
      catch (FileNotFoundException e)
      {
         MQFM.logger.fatal("FileNotFoundException: " + e.getLocalizedMessage());
         MQFM.globalErrorText = e.getLocalizedMessage();
         failed = true;
      }
      catch (IOException e)
      {
         MQFM.logger.fatal("IOException: " + e.getLocalizedMessage());
         MQFM.globalErrorText = e.getLocalizedMessage();
         failed = true;
      }
      catch (Exception e)
      {
         MQFM.logger.fatal("Exception: " + e.getLocalizedMessage());
         MQFM.globalErrorText = e.getLocalizedMessage();
         failed = true;
      }
      finally
      {
         try
         {
            if(fileLock != null)
               fileLock.release();
         }
         catch (IOException e)
         {
            MQFM.logger.fatal("IOException: " + e.getLocalizedMessage());
            failed = true;
         }

         try
         {
            if (fileChannel != null)
               fileChannel.close();
         }
         catch (IOException e)
         {
            MQFM.logger.fatal("IOException: " + e.getLocalizedMessage());
            failed = true;
         }

         try
         {
            if (in != null)
               in.close();
         }
         catch (IOException e)
         {
            MQFM.logger.fatal("IOException: " + e.getLocalizedMessage());
            failed = true;
         }

         if (failed == true)
            throw new MQFMException("IO.readFile had an error. Please check log for more details.");
      }

      return byteData;
   }

   /**
    * This method will write a file.
    *
    * @param fileName the name of the file.
    * @param logger the Log4J logger
    * @param data the data to be written.
    * @throws MQFMException
    */
   public static void writeFile(String fileName, byte[] data, boolean append) throws MQFMException
   {
      RandomAccessFile out = null;
      FileChannel fileChannel = null;
      FileLock fileLock = null;
      boolean failed = false;
      MQFM.logger.debug("writeFile()");

      /* Get rid of the file if it exists. */
      if (new File(fileName).exists())
         deleteFile(fileName);
      
      try
      {
         out = new RandomAccessFile(fileName, "rw");

         if (MQFM.useFileLocking)
         {
            fileChannel = out.getChannel();
            fileLock = fileChannel.tryLock(); /* exclusive lock for writing */
         }

         if ( ( (fileLock != null) && (MQFM.useFileLocking) ) ||
              (!MQFM.useFileLocking) )
         {
            if (append)
            {
               long fileLen = out.length();
               out.seek(fileLen);   // move to the end of the file.
            }
            out.write(data);
            MQFM.logger.info("Successfully completed write to file: "+fileName);
         }
         else
         {
            MQFM.logger.fatal("IO.writeFile cannot acquire lock for file: " + fileName);
            MQFM.globalErrorText = "IO.writeFile cannot acquire lock for file: " + fileName;
         }
      }
      catch (FileNotFoundException e)
      {
         MQFM.logger.fatal("FileNotFoundException: " + e.getLocalizedMessage());
         MQFM.globalErrorText = e.getLocalizedMessage();
         failed = true;
      }
      catch (IOException e)
      {
         MQFM.logger.fatal("IOException: " + e.getLocalizedMessage());
         MQFM.globalErrorText = e.getLocalizedMessage();
         failed = true;
      }
      catch (Exception e)
      {
         MQFM.logger.fatal("Exception: " + e.getLocalizedMessage());
         MQFM.globalErrorText = e.getLocalizedMessage();
         failed = true;
      }
      finally
      {
         try
         {
            if(fileLock != null)
               fileLock.release();
         }
         catch (IOException e)
         {
            MQFM.logger.fatal("IOException: " + e.getLocalizedMessage());
            failed = true;
         }

         try
         {
            if (fileChannel != null)
               fileChannel.close();
         }
         catch (IOException e)
         {
            MQFM.logger.fatal("IOException: " + e.getLocalizedMessage());
            failed = true;
         }
         try
         {
            if (out != null)
               out.close();
         }
         catch (IOException e)
         {
            MQFM.logger.fatal("IOException: " + e.getLocalizedMessage());
            failed = true;
         }

         if (failed == true)
            throw new MQFMException("IO.writeFile had an error. Please check log for more details.");
      }

      return;
   }

   /**
    * Append 1 file to the end of another file.
    * @param srcFileName
    * @param dstFileName
    * @throws MQFMException
    */
   public static void appendFile(String srcFileName, String dstFileName) throws MQFMException
   {
      FileChannel srcChannel = null;
      FileChannel dstChannel = null;
      FileLock srcLock = null;
      FileLock dstLock = null;
      boolean failed = false;
      long count = 0;
      long position = 0;
      MQFM.logger.debug("appendFile()");

      try
      {
         // Create channel on the target
         dstChannel = new FileOutputStream(dstFileName, true).getChannel();
         if (MQFM.useFileLocking)
         {
            dstLock = dstChannel.tryLock(); /* exclusive lock for writing */
         }
         position = dstChannel.size();

         // Create channel on the source
         srcChannel = new FileInputStream(srcFileName).getChannel();
         count = srcChannel.size();
         if (MQFM.useFileLocking)
         {
            srcLock = srcChannel.tryLock(0, count, true); /* shared lock for reading */
         }

         if ( ( (dstLock != null) && (srcLock != null) && (MQFM.useFileLocking) ) ||
              (!MQFM.useFileLocking) )
         {
            // Copy file contents from source to destination
            dstChannel.transferFrom(srcChannel, position, count);

            MQFM.logger.info("Successfully appended file: "+srcFileName+ " to " + dstFileName);
         }
         else
         {
            String s;
            if (dstLock != null)
               s = "IO.appendFile cannot acquire lock for file: " + dstFileName;
            else
               s = "IO.appendFile cannot acquire lock for file: " + srcFileName;

            MQFM.logger.fatal(s);
            MQFM.globalErrorText = s;
         }
      }
      catch (FileNotFoundException e)
      {
         MQFM.logger.fatal("FileNotFoundException: " + e.getLocalizedMessage());
         MQFM.globalErrorText = e.getLocalizedMessage();
         failed = true;
      }
      catch (IOException e)
      {
         MQFM.logger.fatal("IOException: " + e.getLocalizedMessage());
         MQFM.globalErrorText = e.getLocalizedMessage();
         failed = true;
      }
      catch (Exception e)
      {
         MQFM.logger.fatal("Exception: " + e.getLocalizedMessage());
         MQFM.globalErrorText = e.getLocalizedMessage();
         failed = true;
      }
      finally
      {
         try
         {
            if(dstLock != null)
               dstLock.release();
         }
         catch (IOException e)
         {
            MQFM.logger.fatal("IOException: " + e.getLocalizedMessage());
            failed = true;
         }

         try
         {
            if(srcLock != null)
               srcLock.release();
         }
         catch (IOException e)
         {
            MQFM.logger.fatal("IOException: " + e.getLocalizedMessage());
            failed = true;
         }

         // Close the channels
         try
         {
            if (srcChannel != null)
               srcChannel.close();
         }
         catch (IOException e)
         {
            MQFM.logger.fatal("IOException: " + e.getLocalizedMessage());
            failed = true;
         }

         try
         {
            if (dstChannel != null)
               dstChannel.close();
         }
         catch (IOException e)
         {
            MQFM.logger.fatal("IOException: " + e.getLocalizedMessage());
            failed = true;
         }

         if (failed == true)
            throw new MQFMException("IO.appendFile had an error. Please check log for more details.");
      }
   }

   /**
    * Copy the specified file.
    * @param srcFileName
    * @param dstFileName
    * @throws MQFMException
    */
   public static void copyFile(String srcFileName, String dstFileName) throws MQFMException
   {
      FileChannel srcChannel = null;
      FileChannel dstChannel = null;
      FileLock srcLock = null;
      FileLock dstLock = null;
      long len;
      boolean failed = false;
      MQFM.logger.debug("copyFile()");

      try
      {
         // Create channel on the source
         srcChannel = new FileInputStream(srcFileName).getChannel();
         len = srcChannel.size();
         if (MQFM.useFileLocking)
         {
            srcLock = srcChannel.tryLock(0, len, true); /* shared lock for reading */
         }

         // Create channel on the destination
         dstChannel = new FileOutputStream(dstFileName).getChannel();
         if (MQFM.useFileLocking)
         {
            dstLock = dstChannel.tryLock(); /* exclusive lock for writing */
         }

         if ( ( (dstLock != null) && (srcLock != null) && (MQFM.useFileLocking) ) ||
              (!MQFM.useFileLocking) )
         {
            // Copy file contents from source to destination
            dstChannel.transferFrom(srcChannel, 0, srcChannel.size());

            MQFM.logger.info("Successfully copied file: "+srcFileName+ " to " + dstFileName);
         }
         else
         {
            String s;
            if (dstLock != null)
               s = "IO.copyFile cannot acquire lock for file: " + dstFileName;
            else
               s = "IO.copyFile cannot acquire lock for file: " + srcFileName;

            MQFM.logger.fatal(s);
            MQFM.globalErrorText = s;
         }
      }
      catch (FileNotFoundException e)
      {
         MQFM.logger.fatal("FileNotFoundException: " + e.getLocalizedMessage());
         MQFM.globalErrorText = e.getLocalizedMessage();
         failed = true;
      }
      catch (IOException e)
      {
         MQFM.logger.fatal("IOException: " + e.getLocalizedMessage());
         MQFM.globalErrorText = e.getLocalizedMessage();
         failed = true;
      }
      catch (Exception e)
      {
         MQFM.logger.fatal("Exception: " + e.getLocalizedMessage());
         MQFM.globalErrorText = e.getLocalizedMessage();
         failed = true;
      }
      finally
      {
         try
         {
            if(dstLock != null)
               dstLock.release();
         }
         catch (IOException e)
         {
            MQFM.logger.fatal("IOException: " + e.getLocalizedMessage());
            failed = true;
         }

         try
         {
            if(srcLock != null)
               srcLock.release();
         }
         catch (IOException e)
         {
            MQFM.logger.fatal("IOException: " + e.getLocalizedMessage());
            failed = true;
         }
         // Close the channels
         try
         {
            if (srcChannel != null)
               srcChannel.close();
         }
         catch (IOException e)
         {
            MQFM.logger.fatal("IOException: " + e.getLocalizedMessage());
            failed = true;
         }

         try
         {
            if (dstChannel != null)
               dstChannel.close();
         }
         catch (IOException e)
         {
            MQFM.logger.fatal("IOException: " + e.getLocalizedMessage());
            failed = true;
         }

         if (failed == true)
            throw new MQFMException("IO.copyFile had an error. Please check log for more details.");
      }
   }

   /**
    * Copy the specified file from a URL.
    * @param urlFileName
    * @param dstFileName
    * @throws MQFMException
    */
   public static void copyFileFromURL(String urlFileName, String dstFileName) throws MQFMException
   {
      FileChannel dstChannel = null;
      FileLock dstLock = null;
      boolean failed = false;
      URL url = null;
      MQFM.logger.debug("copyFileFromURL()");

      try
      {
         // Create channel on the destination
         dstChannel = new FileOutputStream(dstFileName).getChannel();
         if (MQFM.useFileLocking)
         {
            dstLock = dstChannel.tryLock(); /* exclusive lock for writing */
         }

         if ( ( (dstLock != null) && (MQFM.useFileLocking) ) ||
              (!MQFM.useFileLocking) )
         {
            // Copy URL file contents from source to destination
            url = new URL(urlFileName);
            ReadableByteChannel rbc = Channels.newChannel(url.openStream());
            dstChannel.transferFrom(rbc, 0, 1 << 24);

            MQFM.logger.info("Successfully copied file: "+urlFileName+ " to " + dstFileName);
         }
         else
         {
            String s = "IO.copyFileFromURL cannot acquire lock for file: " + dstFileName;
            MQFM.logger.fatal(s);
            MQFM.globalErrorText = s;
         }
      }
      catch (FileNotFoundException e)
      {
         MQFM.logger.fatal("FileNotFoundException: " + e.getLocalizedMessage());
         MQFM.globalErrorText = e.getLocalizedMessage();
         failed = true;
      }
      catch (IOException e)
      {
         MQFM.logger.fatal("IOException: " + e.getLocalizedMessage());
         MQFM.globalErrorText = e.getLocalizedMessage();
         failed = true;
      }
      catch (Exception e)
      {
         MQFM.logger.fatal("Exception: " + e.getLocalizedMessage());
         MQFM.globalErrorText = e.getLocalizedMessage();
         failed = true;
      }
      finally
      {
         try
         {
            if(dstLock != null)
               dstLock.release();
         }
         catch (IOException e)
         {
            MQFM.logger.fatal("IOException: " + e.getLocalizedMessage());
            failed = true;
         }

         try
         {
            if (dstChannel != null)
               dstChannel.close();
         }
         catch (IOException e)
         {
            MQFM.logger.fatal("IOException: " + e.getLocalizedMessage());
            failed = true;
         }

         if (failed == true)
            throw new MQFMException("IO.copyFileFromURL had an error. Please check log for more details.");
      }
   }

   /**
    * Merge 2 or more files to a target file.
    * @param srcFileNames
    * @param dstFileName
    * @throws MQFMException
    */
   public static void mergeFiles(String[] srcFileNames, String dstFileName) throws MQFMException
   {
      FileChannel srcChannel = null;
      FileChannel dstChannel = null;
      FileLock srcLock = null;
      FileLock dstLock = null;
      boolean failed = false;
      long count = 0;
      long position = 0;
      MQFM.logger.debug("mergeFiles()");

      try
      {
         // Create channel on the destination
         dstChannel = new FileOutputStream(dstFileName).getChannel();
         if (MQFM.useFileLocking)
         {
            dstLock = dstChannel.tryLock(); /* exclusive lock for writing */
         }

         if ( ( (dstLock != null) && (MQFM.useFileLocking) ) ||
              (!MQFM.useFileLocking) )
         {
            for (int i=0; i < srcFileNames.length; i++)
            {
               // Create channel on the source
               srcChannel = new FileInputStream(srcFileNames[i]).getChannel();
               count = srcChannel.size();
               if (MQFM.useFileLocking)
               {
                  srcLock = srcChannel.tryLock(0, count, true); /* shared lock for reading */
               }

               if ( ( (srcLock != null) && (MQFM.useFileLocking) ) ||
                    (!MQFM.useFileLocking) )
               {
                  // Copy file contents from source to destination
                  dstChannel.transferFrom(srcChannel, position, count);

                  position += count;

                  if(srcLock != null)
                     srcLock.release();

                  // Close the channels
                  srcChannel.close();
               }
               else
               {
                  MQFM.logger.fatal("IO.mergeFiles cannot acquire lock for file: " + srcFileNames[i]);
                  MQFM.globalErrorText = "IO.mergeFiles cannot acquire lock for file: " + srcFileNames[i];
               }
            }
         }
         else
         {
            MQFM.logger.fatal("IO.mergeFiles cannot acquire lock for file: " + dstFileName);
            MQFM.globalErrorText = "IO.mergeFiles cannot acquire lock for file: " + dstFileName;
         }
      }
      catch (FileNotFoundException e)
      {
         MQFM.logger.fatal("FileNotFoundException: " + e.getLocalizedMessage());
         MQFM.globalErrorText = e.getLocalizedMessage();
         failed = true;
      }
      catch (IOException e)
      {
         MQFM.logger.fatal("IOException: " + e.getLocalizedMessage());
         MQFM.globalErrorText = e.getLocalizedMessage();
         failed = true;
      }
      catch (Exception e)
      {
         MQFM.logger.fatal("Exception: " + e.getLocalizedMessage());
         MQFM.globalErrorText = e.getLocalizedMessage();
         failed = true;
      }
      finally
      {
         try
         {
            if(dstLock != null)
               dstLock.release();
         }
         catch (IOException e)
         {
            MQFM.logger.fatal("IOException: " + e.getLocalizedMessage());
            failed = true;
         }

         // Close the channel
         try
         {
            if (dstChannel != null)
               dstChannel.close();
         }
         catch (IOException e)
         {
            MQFM.logger.fatal("IOException: " + e.getLocalizedMessage());
            failed = true;
         }

         if (failed == true)
            throw new MQFMException("IO.mergeFiles had an error. Please check log for more details.");
      }
   }

   /**
    * Delete a file.
    * @param completePathAndFileName
    * @param logger
    * @throws MQFMException
    */
   public static boolean deleteFile(String completePathAndFileName) throws MQFMException
   {
      boolean flag = false;
      MQFM.logger.debug("deleteFile()");

      File file = new File(completePathAndFileName);
      if (file.delete())
      {
         MQFM.logger.info("Successfully deleted file: "+completePathAndFileName);
         flag = true;
      }
      else
      {
         MQFM.logger.fatal("Error: Cannot delete file: "+completePathAndFileName);
         flag = false;
         throw new MQFMException("IO.deleteFile had an error. Please check log for more details.");
      }

      return flag;
   }

   /**
    * Deletes all files and subdirectories under dir.
    * Returns true if all deletions were successful.
    * If a deletion fails, the method stops attempting to delete and returns false.
    * @param file
    * @return flag boolean successful or failed
    */
   public static boolean deleteDir(File dir)
   {
      MQFM.logger.debug("deleteDir()");
      
       if (dir.isDirectory())
       {
           String[] children = dir.list();
           for (int i=0; i<children.length; i++)
           {
               boolean success = deleteDir(new File(dir, children[i]));
               if (!success)
               {
                   return false;
               }
           }
       }

       // The directory is now empty so delete it
       return dir.delete();
   }

   /**
    * Touch a file.
    * @param completePathAndFileName
    * @param logger
    * @throws MQFMException
    */
   public static boolean touchFile(String completePathAndFileName) throws MQFMException
   {
      boolean flag = false;
      MQFM.logger.debug("touchFile()");

      File f = new File(completePathAndFileName);

      if (f.exists())
      {
         flag = f.setLastModified((new Date()).getTime());
         MQFM.logger.info("Successfully touched file: "+completePathAndFileName);
      }
      else
      {
         try
         {
            flag = f.createNewFile();
            MQFM.logger.info("Successfully touched (created) file: "+completePathAndFileName);
         }
         catch(IOException ioe)
         {
            MQFM.logger.fatal("Error: Cannot touch (create) file: "+completePathAndFileName + " : "+ioe.getLocalizedMessage());
            flag = false;
            throw new MQFMException("IO.touchFile had an error. Please check log for more details.");
         }
      }

      return flag;
   }

   /**
    * Archive a file.
    * @param completePathAndFileName
    * @param logger
    * @throws MQFMException
    */
   public static void archiveFile(String completePathAndFileName, String targetDir, String targetFilename) throws MQFMException
   {
      MQFM.logger.debug("archiveFile()");

      String fileName = getFileName(completePathAndFileName);
      String targetCompletePathAndFileName = apendFileToPath(targetDir, (targetFilename!=null?targetFilename:fileName) );

      try
      {
         copyFile(completePathAndFileName, targetCompletePathAndFileName);
      }
      catch (MQFMException e)
      {
         throw new MQFMException(e.getLocalizedMessage());
      }

      deleteFile(completePathAndFileName);
   }

   /**
    * Check an string array of file names for any with a wildcard.  Create a new string array
    * of the file names.
    * @param srcFilenames
    * @return
    * @throws MQFMException
    */
   public static String[] buildStringArrayOfFiles(String[] srcFilenames) throws MQFMException
   {
      ArrayList<String> al = new ArrayList<String>();
      MQFM.logger.debug("buildStringArrayOfFiles()");

      for (int i=0; i < srcFilenames.length; i++)
      {
         if (srcFilenames[i].indexOf('*') != -1)
         {
            try
            {
               File[] fileList = IO.getFilteredFileList(srcFilenames[i]);
               for (int j=0; j < fileList.length; j++)
               {
                  al.add(fileList[j].toString());
               }
            }
            catch (IOException e)
            {
               MQFM.logger.fatal("IOException: "+e.getLocalizedMessage());
               if (e.getCause() != null)
                  MQFM.logger.fatal(e.getCause().toString());
               throw new MQFMException("IO.buildStringArrayOfFiles had an error. Please check log for more details.");
            }
         }
         else
         {
            al.add(srcFilenames[i]);
         }
      }

      /**
       * Create string array of the new filenames.
       */
      String[] newFilenames = new String[al.size()];

      for (int i=0; i < al.size(); i++)
         newFilenames[i] = al.get(i);

      return newFilenames;
   }

   /**
    * Retrieve just the filename of a string that may contain both path and filename.
    * @param completePathAndFileName
    * @return fileName
    */
   public static String getFileName(String completePathAndFileName)
   {
      String fileName;
      MQFM.logger.debug("getFileName()");

      int x = completePathAndFileName.lastIndexOf(File.separator);
      if (x != -1)
         fileName = completePathAndFileName.substring(x+1);
      else
         fileName = completePathAndFileName;

      return fileName;
   }

   /**
    * Retrieve just the Path of a string that may contain both path and filename.
    * @param completePathAndFileName
    * @return fileName
    */
   public static String getPath(String completePathAndFileName)
   {
      String path = null;
      MQFM.logger.debug("getPath()");

      if (completePathAndFileName.indexOf(File.separator) != -1)
         path = completePathAndFileName.substring(0,completePathAndFileName.lastIndexOf(File.separator));

      return path;
   }

   /**
    * Build a complete path and filename and check to make sure it has the
    * appropriate file separator.
    *
    * @param path
    * @param filename
    * @return completePathAndFileName
    */
   public static String apendFileToPath(String path, String filename)
   {
      String completePathAndFileName;
      MQFM.logger.debug("apendFileToPath()");

      if (path != null)
      {
         if (path.endsWith(File.separator))
            completePathAndFileName = path + filename;
         else
            completePathAndFileName= path + File.separator + filename;
      }
      else
         completePathAndFileName = filename;

      return completePathAndFileName;
   }

   /**
    * Build a complete path and filename and check to make sure it has the
    * appropriate file separator.
    *
    * @param path
    * @param filename
    * @param separator
    * @return completePathAndFileName
    */
   public static String apendFileToPath(String path, String filename, String separator)
   {
      String completePathAndFileName;
      MQFM.logger.debug("apendFileToPath()");

      if (path != null)
      {
         if (path.endsWith(separator))
            completePathAndFileName = path + filename;
         else
            completePathAndFileName= path + separator + filename;
      }
      else
         completePathAndFileName = filename;

      return completePathAndFileName;
   }

   /**
    * Retrieve a list of files based on wildcard filter.
    *
    * @param pathAndFileName
    * @return fileList
    */
   public static File[] getFilteredFileList(String pathAndFileName) throws IOException
   {
      String pathOnly;
      String filter;
      MQFM.logger.debug("getFilteredFileList()");

      if (pathAndFileName.lastIndexOf(File.separator) != -1)
      {
         pathOnly = pathAndFileName.substring(0,pathAndFileName.lastIndexOf(File.separator));
         filter   = pathAndFileName.substring(pathAndFileName.lastIndexOf(File.separator) + 1);
      }
      else
      {
         pathOnly   = System.getProperty("user.dir");
         filter = pathAndFileName;
      }

      File dir = new File(pathOnly);

      if (!(dir.exists()))
            throw new IOException("Invalid directory : " + pathOnly);

      return dir.listFiles(new FilterFiles(filter));
   }

   /**
    * Copies an input stream to an output stream.
    *
    * @param in
    *           The InputStream.
    * @param out
    *           The OutputStream.
    */
   public static void streamCopier(InputStream in, OutputStream out) throws IOException
   {
      MQFM.logger.debug("streamCopier()");
      
      synchronized (in)
      {
         synchronized (out)
         {
            byte[] buffer = new byte[4096];
            while (true)
            {
               int bytesRead = in.read(buffer);
               if (bytesRead == -1)
                  break;
               out.write(buffer, 0, bytesRead);
            }
         }
      }
   }
}
