package biz.capitalware.mqfm.handlers;

import java.io.File;
import java.io.IOException;
import java.text.DecimalFormat;

import org.jdom.JDOMException;

import biz.capitalware.mqfm.MQFM;
import biz.capitalware.mqfm.action.objects.ReceiveObject;
import biz.capitalware.mqfm.objects.Constants;
import biz.capitalware.mqfm.xml.EmbeddedMsgXml;

import com.ibm.mq.MQMessage;

/**
 * A class to handle remote values in the MQMD or as an XML data in the message payload.
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class RemoteValuesInMsg
{
   private int          xmlDataLength = 0;
   private String       jobFileInMsg = null;
   private String       directoryInMsg = null;
   private String       filenameInMsg = null;
   
   private boolean      _runJob = false;
   private String       _jobXmlFile = null;
   private String       _toPathAndFilename = null;
   private String       _directory = null;

   /**
    * The constructor
    * @param logger
    */
   public RemoteValuesInMsg()
   {
      super();
   }

   /**
    * Get the remote values from the message.
    * @param ro
    * @param receiveMsg
    * @param msgCounter
    * @throws IOException 
    * @throws JDOMException 
    */
   public int processValues(ReceiveObject ro, MQMessage receiveMsg, int msgCounter) throws JDOMException, IOException
   {
      DecimalFormat df0 = new java.text.DecimalFormat("0000");
      boolean ojn = ro.overrideJobName;
      boolean odn = ro.overrideDirectoryName;
      boolean ofn = ro.overrideFileName;
      _directory = ro.defaultDirectory;
      /*
       * Set default values.
       */
      _runJob = false;
      _jobXmlFile = ro.defaultJobXmlFile;
      _toPathAndFilename = null;
      xmlDataLength = 0;
      jobFileInMsg = null;
      directoryInMsg = null;
      filenameInMsg = null;

      /* Is the message one of MQFM's? */
      if ((receiveMsg.feedback & Constants.MQFM_MSG) == Constants.MQFM_MSG)
         extractRemoteValues(receiveMsg);
      else
      {
         /* Cannot use values from MQMD */
         ojn = true;
         odn = true;
         ofn = true;
      }

      if (ojn)
      {
         if (ro.defaultJobXmlFile != null)
            _runJob = true;
      }
      else
      {
         _jobXmlFile = jobFileInMsg;
         if ( !("".equals(_jobXmlFile)) && (_jobXmlFile != null) )
            _runJob = true;
      }

      if (!odn)
         _directory = directoryInMsg;

      /**
       * Tokenize just the directory so that the directory value will be consistent
       * for both _directory &  _toPathAndFilename just in case the user uses the 
       * date token.
       */
      _directory = Tokens.replace(_directory, MQFM.userProperties);;
      
      if (ofn)
      {
         if ( (ro.defaultFilename != null) && (!("".equals(ro.defaultFilename))) )
            _toPathAndFilename = IO.apendFileToPath(_directory, ro.defaultFilename);
         else
            _toPathAndFilename = IO.apendFileToPath(_directory, MQFM.WHO_AM_I + "_"+df0.format(msgCounter+1) + ".txt");
      }
      else
         _toPathAndFilename= IO.apendFileToPath(_directory, filenameInMsg);

      _toPathAndFilename = Tokens.replace(ro.actionName, _toPathAndFilename, null, msgCounter+1, MQFM.userProperties);

      if (_runJob)
      {
         _jobXmlFile = Tokens.replace(ro.actionName, _jobXmlFile, MQFM.userProperties);
      }

      return 0;
   }

   /**
    * Extract remote values set by the send action.
    * @param receiveMsg
    * @throws IOException
    * @throws JDOMException
    */
   private void extractRemoteValues(MQMessage receiveMsg) throws IOException, JDOMException
   {
      if ((receiveMsg.feedback & Constants.MQFM_XML_EMBEDDED_MSG) == Constants.MQFM_XML_EMBEDDED_MSG)
      {
         xmlDataLength = (int)(((receiveMsg.correlationId[0] & 0xffL) << 24) |
                                ((receiveMsg.correlationId[1] & 0xffL) << 16) |
                                ((receiveMsg.correlationId[2] & 0xffL) << 8) |
                                 (receiveMsg.correlationId[3] & 0xffL));
         byte[] bx = new byte[xmlDataLength];
         receiveMsg.readFully(bx);

         EmbeddedMsgXml emx = new EmbeddedMsgXml();
         emx.readXMLFile(bx);
         if ((receiveMsg.feedback & Constants.MQFM_START_JOB) == Constants.MQFM_START_JOB)
            jobFileInMsg = emx.getJobFile();

         directoryInMsg = emx.getDirectory();
         filenameInMsg = emx.getFilename();
      }
      else
      {
         if ((receiveMsg.feedback & Constants.MQFM_START_JOB) == Constants.MQFM_START_JOB)
            jobFileInMsg = (new String(receiveMsg.correlationId)).trim() + Constants.XML_SUFFIX;

         directoryInMsg = (new String(receiveMsg.accountingToken)).trim();
         filenameInMsg = (receiveMsg.applicationIdData).trim();
      }

      if ( !(directoryInMsg.endsWith(File.separator)) )
         directoryInMsg += File.separator;

      MQFM.logger.info("Remote values:");
      MQFM.logger.info("    JobFile             = " + (jobFileInMsg == null?"":jobFileInMsg));
      MQFM.logger.info("    Directory           = " + directoryInMsg);
      MQFM.logger.info("    File Name           = " + filenameInMsg);
   }

   /**
    * Run the job or not.
    * @return runJob
    */
   public boolean isRunJob()
   {
      return _runJob;
   }

   /**
    * Retrieve the Job XML File.
    * @return jobXmlFile
    */
   public String getJobXmlFile()
   {
      return _jobXmlFile;
   }

   /**
    * Retrieve the complete path and filename.
    * @return toPathAndFilename
    */
   public String getToPathAndFilename()
   {
      return _toPathAndFilename;
   }

   /**
    * Retrieve the directory
    * @return directory
    */
   public String getDirectory()
   {
      return _directory;
   }

   /**
    * Retrieve the XML data data length
    * @return xmlDataLength
    */
   public int getXmlDataLength()
   {
      return xmlDataLength;
   }
}
