package biz.capitalware.mqfm.mq;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.security.InvalidAlgorithmParameterException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertStore;
import java.security.cert.LDAPCertStoreParameters;
import java.util.ArrayList;
import java.util.Hashtable;

import biz.capitalware.mqfm.MQFM;
import biz.capitalware.mqfm.mq.objects.QueueObject;
import biz.capitalware.mqfm.mq.objects.MQObject;
import biz.capitalware.mqfm.objects.MQFMException;

import com.ibm.mq.MQC;
import com.ibm.mq.MQDistributionList;
import com.ibm.mq.MQDistributionListItem;
import com.ibm.mq.MQException;
import com.ibm.mq.MQQueue;
import com.ibm.mq.MQQueueManager;
import com.ibm.mq.MQSecurityExit;

/**
 * A base class to define common MQ handler methods.
 *
 * @author Roger Lacroix, Capitalware Inc.
 * @version 1.3
 * @license Apache 2 License
 */
public class MyMQ extends Thread
{
   protected   MQObject        mqo;
   protected   MQQueueManager  _qMgr = null;

   /**
    * The constructor
    * @param logger
    */
   public MyMQ(MQObject mqo)
   {
      super();
      this.mqo = mqo;
   }

   /**
    * Connect to MQ Server
    *
    * @throws MQException
    * @throws MalformedURLException
    * @throws ClassNotFoundException
    * @throws IllegalAccessException
    * @throws InstantiationException
    * @throws NoSuchAlgorithmException
    * @throws InvalidAlgorithmParameterException
    */
   public void connect() throws MQException, MalformedURLException, ClassNotFoundException, InstantiationException, IllegalAccessException, InvalidAlgorithmParameterException, NoSuchAlgorithmException, Exception
   {
      MQFM.logger.debug("Attempting to connect to queue manager '"+mqo.qMgrName+"'");
      MQException.log = null;
      boolean setUserIDPassword = true;

      if (mqo.channelName != null)
      {
         Hashtable mqht = new Hashtable();

         mqht.put(MQC.CHANNEL_PROPERTY, mqo.channelName);
         mqht.put(MQC.HOST_NAME_PROPERTY, mqo.hostname);
         mqht.put(MQC.PORT_PROPERTY, new Integer(mqo.port));

         if (mqo.securityExit != null)
         {
            if ( ("biz.capitalware.mqausx.MQAUSXJ".equals(mqo.securityExit)) ||
                 ("biz.capitalware.mqausx.MQAUSXJ2EE".equals(mqo.securityExit)) )
            {
               mqht.put(MQC.SECURITY_EXIT_PROPERTY, CWSX.getMQAUSXJ(mqo));
               
               /* Did we already pass the info to MQAUSXJ or MQAUSXJ2EE? */
               if (mqo.securityExitData != null)
                  setUserIDPassword = false;
               else if ( (mqo.userID != null) && (mqo.password != null) )
                  setUserIDPassword = false;
            }
            else
            {
               Class se = getSEClass(mqo.securityExit, mqo.securityExitPath);
               mqht.put(MQC.SECURITY_EXIT_PROPERTY, (MQSecurityExit) se.newInstance());
            }
         }
         
         if (setUserIDPassword)
         {
            if (mqo.userID != null)
               mqht.put(MQC.USER_ID_PROPERTY, mqo.userID);

            if (mqo.password != null)
               mqht.put(MQC.PASSWORD_PROPERTY, mqo.password);
         }

         /*
          * SSL
          */
         if (mqo.cipherSuiteName != null)
            mqht.put(MQC.SSL_CIPHER_SUITE_PROPERTY, mqo.cipherSuiteName);

         if (mqo.distinguishedName != null)
            mqht.put(MQC.SSL_PEER_NAME_PROPERTY, mqo.distinguishedName);

         if (mqo.trustedStore != null)
            System.setProperty("javax.net.ssl.trustStore", mqo.trustedStore);

         if (mqo.trustedStorePasswd != null)
            System.setProperty("javax.net.ssl.trustStorePassword", mqo.trustedStorePasswd);

         if (mqo.keyStore != null)
            System.setProperty("javax.net.ssl.keyStore", mqo.keyStore);

         if (mqo.keyStorePasswd != null)
            System.setProperty("javax.net.ssl.keyStorePassword", mqo.keyStorePasswd);

         if ( ((mqo.ldapServer != null)) && (mqo.ldapServerPort != 0) )
         {
            LDAPCertStoreParameters ldapcsparms = new LDAPCertStoreParameters(mqo.ldapServer, mqo.ldapServerPort);
            CertStore cs = CertStore.getInstance("LDAP", ldapcsparms);
            ArrayList al = new ArrayList();
            al.add(cs);
            mqht.put(MQC.SSL_CERT_STORE_PROPERTY, al);
         }

         _qMgr = new MQQueueManager(mqo.qMgrName, mqht);

         MQFM.logger.info("Successfully connected to " + mqo.qMgrName + " using " + mqo.channelName + " and " + mqo.hostname+"("+mqo.port+")");
      }
      else if (mqo.CCDTFile != null)
      {
         String trustedStore = System.getenv("MQSSLKEYR");
         if (trustedStore != null)
            System.setProperty("javax.net.ssl.trustStore", trustedStore + ".jks");

         File file = new File(mqo.CCDTFile);
         URL url = file.toURL();

         if (mqo.userID != null)
         {
            Hashtable mqht = new Hashtable();

            if (mqo.userID != null)
               mqht.put(MQC.USER_ID_PROPERTY, mqo.userID);

            if (mqo.password != null)
               mqht.put(MQC.PASSWORD_PROPERTY, mqo.password);

            _qMgr = new MQQueueManager(mqo.qMgrName, mqht, url);

            MQFM.logger.info("Successfully connected to " + mqo.qMgrName + " using " + mqo.CCDTFile + " with UserID");
         }
         else
         {
            _qMgr = new MQQueueManager(mqo.qMgrName, url);

            MQFM.logger.info("Successfully connected to " + mqo.qMgrName + " using " + mqo.CCDTFile);
         }
      }
      else
      {
         _qMgr = new MQQueueManager(mqo.qMgrName);

         MQFM.logger.info("Successfully connected to " + mqo.qMgrName);
      }
   }

   /**
    *
    * @param clazzName
    * @param pathToJar
    * @return
    */
   private Class getSEClass(String clazzName, String pathToJar) throws MalformedURLException, ClassNotFoundException
   {
      Class cls = null;

      // Create a File object on the root of the directory containing the class file
      File file = new File(pathToJar);

      // Convert File to a URL
      URL url = file.toURL();
      URL[] urls = new URL[]
      {
         url
      };

      // Create a new class loader with the directory
      ClassLoader cl = new URLClassLoader(urls);

      // Load in the class; MyClass.class should be located in
      // the directory file:/c:/myclasses/com/mycompany
      cls = cl.loadClass(clazzName);

      return cls;
   }

   /**
    * Disconnect from the queue manager
    */
   public void disconnect()
   {
      try
      {
         if (_qMgr != null)
         {
            MQFM.logger.debug("Attempting to disconnect from queue manager '"+mqo.qMgrName+"'");
            _qMgr.disconnect();
            MQFM.logger.info("Sucessfully disconnected.");
         }
      }
      catch (MQException e)
      {
         MQFM.logger.fatal("MQException: " + e.getMessage());
         MQFM.logger.fatal("MQCC=" + e.completionCode + " : MQRC="+e.reasonCode);
      }
   }

   /**
    * Open a temporary dynamic queue
    * @throws MQFMException 
    */
   public MQQueue openTempDynQ() throws MQFMException
   {
      String inputQName   = "SYSTEM.DEFAULT.MODEL.QUEUE";
      int openInputOptions  = MQC.MQOO_INQUIRE + MQC.MQOO_FAIL_IF_QUIESCING + MQC.MQOO_INPUT_SHARED;
      String DynamicQName = "CAPITALWARE.MQFM.*";
      MQQueue inQ = null;
      
      try
      {
         MQFM.logger.debug("Attempting to create/open temporary dynamic queue '"+DynamicQName+"'");
         
         inQ = _qMgr.accessQueue( inputQName,
                                  openInputOptions,
                                  null,           // default q manager
                                  DynamicQName,   // dynamic q name
                                  null );

         MQFM.logger.info("Successfully opened temporary dynamic queue: '" + inQ.name + "'");
      }
      catch (MQException e)
      {
         MQFM.logger.fatal("MQException: " + e.getLocalizedMessage() + " : QName=" + inputQName + " : DynamicQName="+DynamicQName);
         MQFM.logger.fatal("MQCC=" + e.completionCode + " : MQRC="+e.reasonCode);
         MQFM.globalErrorText = e.getLocalizedMessage();
         inQ = null;
         throw new MQFMException("openTempDynQ: MQCC=" + e.completionCode + " : MQRC="+e.reasonCode+" : QName=" + inputQName + " : DynamicQName="+DynamicQName);
      }

      return inQ;
   }

   /**
    * Open the input queue
    *
    * @param qObject
    * @throws MQFMException 
    * @throws MQException
    */
   public MQQueue openInQ(QueueObject qObject) throws MQFMException
   {
      MQQueue inQ = null;
      try
      {
         MQFM.logger.debug("Attempting to open input queue '"+qObject.qName+"'");
         
         inQ = _qMgr.accessQueue( qObject.qName,
                                  MQC.MQOO_INPUT_AS_Q_DEF + MQC.MQOO_FAIL_IF_QUIESCING,
                                  null,
                                  null,
                                  null );
         MQFM.logger.info("Successfully opened input queue: '" + qObject + "'");
      }
      catch (MQException e)
      {
         MQFM.logger.fatal("MQException: " + e.getLocalizedMessage() + " : QObject=" + qObject);
         MQFM.logger.fatal("MQCC=" + e.completionCode + " : MQRC="+e.reasonCode);
         MQFM.globalErrorText = e.getLocalizedMessage();
         inQ = null;
         throw new MQFMException("openInQ: MQCC=" + e.completionCode + " : MQRC="+e.reasonCode+ " : QObject=" + qObject);
      }

      return inQ;
   }

   /**
    * Open the output queue
    *
    * @param qObject
    * @throws MQFMException 
    * @throws MQException
    */
   public MQQueue openOutQ(QueueObject qObject) throws MQFMException
   {
      MQQueue outQ = null;
      try
      {
         MQFM.logger.debug("Attempting to open output queue '"+qObject.qName+"'");
         
         outQ = _qMgr.accessQueue( qObject.qName,
                                   MQC.MQOO_OUTPUT + MQC.MQOO_SET_IDENTITY_CONTEXT + MQC.MQOO_FAIL_IF_QUIESCING,
                                   qObject.qMgrName,
                                   null,
                                   null );
         MQFM.logger.info("Successfully opened output queue: '" + qObject + "'");
      }
      catch (MQException e)
      {
         MQFM.logger.fatal("MQException: " + e.getLocalizedMessage() + " : QObject=" + qObject);
         MQFM.logger.fatal("MQCC=" + e.completionCode + " : MQRC="+e.reasonCode);
         MQFM.globalErrorText = e.getLocalizedMessage();
         outQ = null;
         throw new MQFMException("openOutQ: MQCC=" + e.completionCode + " : MQRC="+e.reasonCode+ " : QObject=" + qObject);
      }

      return outQ;
   }

   /**
    * Close the queue.
    *
    * @param qHandle
    * @throws MQFMException 
    */
   public void closeQ(MQQueue qHandle)
   {
      try
      {
         if (qHandle != null)
         {
            String qName = qHandle.name.trim();
            
            MQFM.logger.debug("Attempting to close queue '"+qName+"'");
            
            qHandle.close();
            MQFM.logger.info("Successfully closed the queue '"+qName+"'");
         }
      }
      catch (MQException e)
      {
         MQFM.logger.fatal("MQException: " + "MQCC=" + e.completionCode + " : MQRC="+e.reasonCode);
         MQFM.logger.fatal("MQException: " + e.getLocalizedMessage());
         MQFM.globalErrorText = e.getLocalizedMessage();
      }
   }

   /**
    * Open the output distribution list
    *
    * @param ArrayList<MQDistListObject> disList
    * @throws MQFMException 
    * @throws MQException
    */
   public MQDistributionList openOutDL(ArrayList<QueueObject> disList) throws MQFMException
   {
      MQDistributionList outDL = null;
      MQDistributionListItem[] mqdlis = new MQDistributionListItem[disList.size()];
      QueueObject mqdlo = null;
      
      for (int i=0; i < disList.size(); i++)
      {
         mqdlo =  disList.get(i);
         mqdlis[i] = new MQDistributionListItem();
         mqdlis[i].queueName = mqdlo.qName;
         if (mqdlo.qMgrName != null)
            mqdlis[i].queueManagerName = mqdlo.qMgrName;
      }
      
      try
      {
         MQFM.logger.debug("Attempting to open distribution list queue '"+disList.toString()+"'");
         
         outDL = _qMgr.accessDistributionList( mqdlis, MQC.MQOO_OUTPUT + MQC.MQOO_SET_IDENTITY_CONTEXT + MQC.MQOO_FAIL_IF_QUIESCING);
         MQFM.logger.info("Successfully opened distribution list of queues: '" + disList.toString() + "'");
      }
      catch (MQException e)
      {
         MQFM.logger.fatal("MQException: " + e.getLocalizedMessage() + " : distribution list of queues '" + disList.toString()+"'");
         MQFM.logger.fatal("MQCC=" + e.completionCode + " : MQRC="+e.reasonCode);
         MQFM.globalErrorText = e.getLocalizedMessage();
         outDL = null;
         throw new MQFMException("openOutQ: MQCC=" + e.completionCode + " : MQRC="+e.reasonCode+ " : distribution list of queues '" + disList.toString()+"'");
      }

      return outDL;
   }

   /**
    * Close the distribution list.
    *
    * @param dlHandle
    * @throws MQFMException 
    */
   public void closeDL(MQDistributionList dlHandle)
   {
      try
      {
         if (dlHandle != null)
         {
            MQFM.logger.debug("Attempting to close distribution list queue '"+mqo.disListOfQNames+"'");
            
            dlHandle.close();
            MQFM.logger.info("Successfully closed the distribution list '"+mqo.disListOfQNames+"'");
         }
      }
      catch (MQException e)
      {
         MQFM.logger.fatal("MQException: " + "MQCC=" + e.completionCode + " : MQRC="+e.reasonCode);
         MQFM.logger.fatal("MQException: " + e.getLocalizedMessage());
         MQFM.globalErrorText = e.getLocalizedMessage();
      }
   }

   /**
    * Handle backing out the message(s).
    */
   public void backout()
   {
      try
      {
         MQFM.logger.debug("Issuing backout." );
         _qMgr.backout();
         MQFM.logger.info("Issued backout." );
      }
      catch (MQException e)
      {
         MQFM.logger.fatal("MQException: " + e.getMessage() + " : QObject=" + mqo.qObject);
         MQFM.logger.fatal("MQCC=" + e.completionCode + " : MQRC=" + e.reasonCode);
         MQFM.globalErrorText = e.getLocalizedMessage();
      }
   }
}
