package biz.capitalware.mqfm.xml;

import java.util.*;
import java.io.*;

import org.jdom.*;
import org.jdom.input.*;

import com.ibm.mq.MQC;

import biz.capitalware.mqfm.MQFM;
import biz.capitalware.mqfm.action.objects.SendObject;
import biz.capitalware.mqfm.action.objects.WatchObject;
import biz.capitalware.mqfm.handlers.IO;
import biz.capitalware.mqfm.handlers.StringUtilities;
import biz.capitalware.mqfm.handlers.Tokens;
import biz.capitalware.mqfm.objects.Constants;
import biz.capitalware.mqfm.watch.DirectoryWatch;
import biz.capitalware.mqfm.watch.FileWatch;
import biz.capitalware.mqfm.watch.Watch;

/**
 * This class handles the reading and parsing of Watch XML file and parse parameters
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class WatchXml extends FMXml
{
   private int          _pollInterval;
   private Watch[]      _watches;
   private WatchObject  _wo;

   /**
    * The constructor
    * @param logger
    */
   public WatchXml()
   {
      super();
   }

   /**
    * Set the WatchObject to be used to set the default values.
    * @param wo
    */
   public void setWatchObject(WatchObject wo)
   {
      this._wo = wo;
   }

   /**
    * Read Watch XML file and set parameters
    *
    * @param xmlFilename
    *           Filename of test_watch.xml file
    * @throws IOException
    */
   public void readXMLFile(String xmlFilename) throws JDOMException, IOException
   {
      SAXBuilder builder = new SAXBuilder(true);
      Document doc = builder.build(Constants.WATCH_PATH + xmlFilename);
      Element rootElement = doc.getRootElement();

      String pollIntervalS = rootElement.getChild(Constants.POLL_INTERVAL_ELEMENT).getTextTrim();
      _pollInterval = StringUtilities.stringToInt(pollIntervalS, 15); 
      if (_pollInterval < 5)
         throw new JDOMException("The PollInterval must be bigger than 5 seconds");
      
      // read and parse watches
      List watchList = rootElement.getChildren(Constants.WATCH_ELEMENT);
      _watches = new Watch[watchList.size()];
      int wi = 0;
      Iterator it = watchList.iterator();
      while (it.hasNext())
      {
         Element watchElement = (Element) it.next();
         String type = watchElement.getAttribute(Constants.TYPE_ATTRIBUTE).getValue();

         // create watch objects
         Watch t_watch = null;
         // file type of watch: create FileWatch instance
         if ("F".equalsIgnoreCase(type))
         {
            Element fileElement = watchElement.getChild(Constants.FILE_ELEMENT);
            String filename = fileElement.getTextTrim();

            String watchfilename;

            if (fileElement.getAttribute(Constants.DIR_ATTRIBUTE) != null)
               watchfilename = IO.apendFileToPath(fileElement.getAttribute(Constants.DIR_ATTRIBUTE).getValue(), filename);
            else
               watchfilename = filename;

            watchfilename = Tokens.replace(watchfilename, MQFM.userProperties);

            t_watch = new FileWatch(watchfilename);
         }
         // directory watch type: parse all extensions and create DirectoryWatch instance
         else if ("D".equalsIgnoreCase(type))
         {
            Element directoryElement = watchElement.getChild(Constants.DIRECTORY_ELEMENT);
            String watchDirectory = directoryElement.getTextTrim();

            boolean sendFlag = false;
            if (directoryElement.getAttribute(Constants.SEND_ON_STARTUP).getValue().toUpperCase().charAt(0) == 'Y')
               sendFlag = true;
            else
               sendFlag = false;

            // check if file exists:
            File file = new File(watchDirectory);
            if (!file.exists())
               throw new FileNotFoundException("Watch: " + watchDirectory + " not found!");

            List extensionList = watchElement.getChildren(Constants.EXTENSION_ELEMENT);
            String[] filter;
            /* Did the user specify any extensions? */
            if (extensionList.size() > 0)
            {
               filter = new String[extensionList.size()];
               Iterator it2 = extensionList.iterator();
               int f = 0;
               while (it2.hasNext())
               {
                  Element extensionElement = (Element) it2.next();
                  filter[f++] = extensionElement.getTextTrim();
               }
            }
            else
            {  /* set a default if the user forget to specify an extension */
               filter = new String[1];
               filter[0] = "*";
            }

            watchDirectory = Tokens.replace(watchDirectory, MQFM.userProperties);

            t_watch = new DirectoryWatch(watchDirectory, filter, sendFlag);
         }

         t_watch.setSendObject(retrieveWatchSend(watchElement));

         _watches[wi++] = t_watch;
      }
   }

   /**
    * Retrieve all XML values for watch send action.
    * @param watchElement
    * @throws JDOMException
    */
   private SendObject retrieveWatchSend(Element watchElement) throws JDOMException
   {
      Element queueElement;
      SendObject so = new SendObject();
      so.mqXMLFile = _wo.mqXMLFile;
      so.CCDTFile = _wo.CCDTFile;
      so.qMgrName = _wo.qMgrName;
      so.qObject = _wo.qObject;
      so.format = _wo.format;
      so.jobXmlFile = _wo.jobXmlFile;
      so.remoteJobXmlFile = _wo.remoteJobXmlFile;
      so.remoteDirectory = _wo.remoteDirectory;
      so.remoteFilename = _wo.remoteFilename;
      so.archiveDirectory = _wo.archiveDirectory;
      so.archiveFileName = _wo.archiveFileName;
      so.archiveCreateDir = _wo.archiveCreateDir;
      so.priority = _wo.priority;
      so.persistence = _wo.persistence;
      so.deleteFile = _wo.deleteFile;
      so.archiveFile = _wo.archiveFile;
      
      if ( watchElement.getChild(Constants.WATCH_SEND_ELEMENT) != null)
      {
         Element watchSendElement = watchElement.getChild(Constants.WATCH_SEND_ELEMENT);
         
         so.onErrorFail = getOnErrorFailAttribute(watchSendElement, _wo.onErrorFail);

         if ( watchSendElement.getChild(Constants.MQ_ELEMENT) != null)
         {
            Element mqElement = watchSendElement.getChild(Constants.MQ_ELEMENT);

            if ( mqElement.getChild(Constants.MQFILE_ELEMENT) != null)
            {
               so.mqXMLFile = mqElement.getChild(Constants.MQFILE_ELEMENT).getTextTrim();

               boolean exists = (new File(Constants.MQ_PATH + so.mqXMLFile)).exists();
               if ( !(exists) )
                  throw new JDOMException("Error: The MQ XML file does not exists : " + Constants.MQ_PATH + so.mqXMLFile);
            }

            if ( mqElement.getChild(Constants.CCDTFILE_ELEMENT) != null)
               so.CCDTFile = mqElement.getChild(Constants.CCDTFILE_ELEMENT).getTextTrim();

            if ( (so.mqXMLFile != null) && (so.CCDTFile != null) )
               throw new JDOMException("Error: Do not specify both "+Constants.MQFILE_ELEMENT+" and "+Constants.CCDTFILE_ELEMENT+" elements.");

            if ( mqElement.getChild(Constants.QMGRNAME_ELEMENT) != null)
               so.qMgrName = mqElement.getChild(Constants.QMGRNAME_ELEMENT).getTextTrim();
            
            if ( (queueElement = mqElement.getChild(Constants.QUEUENAME_ELEMENT)) != null)
            {
               so.qObject.qName = queueElement.getTextTrim();

               if (queueElement.getAttribute(Constants.QMGRNAME_ATTRIBUTE) != null)
                  so.qObject.qMgrName = queueElement.getAttribute(Constants.QMGRNAME_ATTRIBUTE).getValue().trim();
               else
                  so.qObject.qMgrName = null;
            }

            if (so.mqXMLFile != null)
               so.mqXMLFile = Tokens.replace(so.mqXMLFile, MQFM.userProperties);
            if (so.CCDTFile != null)
               so.CCDTFile = Tokens.replace(so.CCDTFile, MQFM.userProperties);
            if (so.qMgrName != null)
               so.qMgrName = Tokens.replace(so.qMgrName, MQFM.userProperties);
            if (so.qObject != null)
               so.qObject.qName = Tokens.replace(so.qObject.qName, MQFM.userProperties);
            if (so.qMgrName != null)
               so.qObject.qMgrName = Tokens.replace(so.qObject.qMgrName, MQFM.userProperties);
         }

         if ( watchSendElement.getChild(Constants.EXECUTE_ELEMENT) != null)
         {
            Element execElement = watchSendElement.getChild(Constants.EXECUTE_ELEMENT);
            if (execElement.getAttribute(Constants.XML_FILE_ATTRIBUTE) != null)
               so.jobXmlFile = execElement.getAttribute(Constants.XML_FILE_ATTRIBUTE).getValue();
         }

         if ( watchSendElement.getChild(Constants.ARCHIVE_ELEMENT) != null)
         {
            Element archiveElement = watchSendElement.getChild(Constants.ARCHIVE_ELEMENT);

            if (archiveElement.getAttribute(Constants.TO_DIR_ATTRIBUTE) != null)
               so.archiveDirectory = archiveElement.getAttribute(Constants.TO_DIR_ATTRIBUTE).getValue();

            if (archiveElement.getAttribute(Constants.TO_FILE_ATTRIBUTE) != null)
               so.archiveFileName = archiveElement.getAttribute(Constants.TO_FILE_ATTRIBUTE).getValue();

            if (archiveElement.getAttribute(Constants.CREATE_DIR_ATTRIBUTE) != null)
            {
               String flag = archiveElement.getAttribute(Constants.CREATE_DIR_ATTRIBUTE).getValue().toUpperCase();
               if ( (flag.charAt(0) == 'Y') || (flag.charAt(0) == 'y') )
                  so.archiveCreateDir = true;
            }

            so.archiveFile = true;
         }

         if ( watchSendElement.getChild(Constants.REMOTE_ELEMENT) != null)
         {
            Element remoteElement = watchSendElement.getChild(Constants.REMOTE_ELEMENT);

            if ( remoteElement.getChild(Constants.EXECUTE_ELEMENT) != null)
            {
               Element execElement = remoteElement.getChild(Constants.EXECUTE_ELEMENT);
               if (execElement.getAttribute(Constants.XML_FILE_ATTRIBUTE) != null)
                  so.remoteJobXmlFile = execElement.getAttribute(Constants.XML_FILE_ATTRIBUTE).getValue();
            }

            if ( remoteElement.getChild(Constants.DIRECTORY_ELEMENT) != null)
               so.remoteDirectory = remoteElement.getChild(Constants.DIRECTORY_ELEMENT).getTextTrim();

            if ( remoteElement.getChild(Constants.FILENAME_ELEMENT) != null)
               so.remoteFilename = remoteElement.getChild(Constants.FILENAME_ELEMENT).getTextTrim();
         }

         String priority = watchSendElement.getAttribute(Constants.PRIORITY_ATTRIBUTE).getValue();
         so.priority = StringUtilities.stringToInt(priority, MQC.MQPRI_PRIORITY_AS_Q_DEF);

         so.format = watchSendElement.getAttribute(Constants.FORMAT_ATTRIBUTE).getValue().toUpperCase().charAt(0);

         if (watchSendElement.getAttribute(Constants.PERSISTENCE_ATTRIBUTE).getValue().toUpperCase().charAt(0) == 'Y')
            so.persistence = true;
         else
            so.persistence = false;

         if (watchSendElement.getAttribute(Constants.DELETE_ATTRIBUTE).getValue().toUpperCase().charAt(0) == 'Y')
            so.deleteFile = true;
      }

      if (so.srcFilename != null)
         so.srcFilename = Tokens.replace(so.srcFilename, MQFM.userProperties);
      if (so.jobXmlFile != null)
         so.jobXmlFile = Tokens.replace(so.jobXmlFile, MQFM.userProperties);
      if (so.remoteDirectory != null)
         so.remoteDirectory = Tokens.replace(so.remoteDirectory, MQFM.userProperties);
      if (so.remoteFilename != null)
         so.remoteFilename = Tokens.replace(so.remoteFilename, MQFM.userProperties);
      if (so.remoteJobXmlFile != null)
         so.remoteJobXmlFile = Tokens.replace(so.remoteJobXmlFile, MQFM.userProperties);

      return so;
   }

   /**
    * Retrieve the poll interval
    * @return int
    */
   public int getPollInterval()
   {
      return _pollInterval * 1000;
   }

   /**
    * Retrieve an array of watches.
    * @return watch
    */
   public Watch[] getWatches()
   {
      return _watches;
   }
}
